//
//  SBConfigureView.m
//  SixBookmarks
//
//  Created by ss on 11. 6. 17..
//  Copyright 2011 OpenTheGate. All rights reserved.
//

#import "SBConfigureView.h"
#import "MyClasses.h"

#import "SBBookmark.h"

@implementation SBConfigureView


@synthesize bookmarks;
@synthesize table;
@synthesize syncEngine;


-(IBAction)handleClose:(id)sender {
    // are we editing?
	if(self.table.editing)
		[self stopEditing];
    else
		[self.owner openNavigator];
}

-(void)stopEditing
{
	[self.table setEditing:FALSE animated:TRUE];
}


-(IBAction)handleSync:(id)sender {
    if(self.syncEngine == nil)
		self.syncEngine = [[SBSync alloc] init];
	[self.syncEngine doSync:(SBSyncCallback *)self];
}

-(IBAction)handleEdit:(id)sender {
    [self.table setEditing:TRUE animated:TRUE];
}


-(IBAction)handleAdd:(id)sender {
    int ordinal = [self getOrdinalToAdd];
	if(ordinal != -1)
		[self.owner openBookmarkSingleton:ordinal];
	else
		[SBMessageBox show:@"No more bookmarks can be added."];
}


-(int)getOrdinalToAdd
{
	NSMutableArray *array = [NSMutableArray array];
	for(int index = 0; index < 6; index++)
		[array addObject:[NSNumber numberWithBool:FALSE]];
	for(SBBookmark  *bookmark in self.bookmarks)
		[array replaceObjectAtIndex:[bookmark ordinal] withObject:[NSNumber numberWithBool:TRUE]];
    
	// walk...
	for(int index = 0; index < 6; index++)
	{
		NSNumber *value = [array objectAtIndex:index];
		if([value boolValue] == FALSE)
			return index;
	}
	
	// return...
	return -1;
}


-(void)syncOk
{
	[self refreshView];
	[SBMessageBox show:@"Sync OK."];
}

-(void)syncFailed:(NSError *)err
{
	// nope...
	[SBMessageBox showError:err];
}



-(void)refreshView
{
    // do we have bookmarks?
    BOOL doReload = FALSE;
    if (self.bookmarks != nil)
        doReload = TRUE;
    
    // get our bookmarks...
    NSMutableArray *theBookmarks = nil;
    NSError *err = [SBBookmark getBookmarksForDisplay:&theBookmarks];
    if (err != nil)
        [SBMessageBox showError:err];
    else
    {
        // keep track...
        self.bookmarks = theBookmarks;
        
        // sort by ordinal...
        [theBookmarks sortUsingSelector:@selector(ordinalComparer:)];
    }
    
    // reload...
    if (doReload)
        [self.table reloadData];
}


-(NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [self.bookmarks count];
}

-(UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *cellId = @"BookmarkKey";
    
    // find one...
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellId];
    if (cell == nil)
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellId] autorelease];
    
    // set...
    NSUInteger row = [indexPath row];
    cell.textLabel.text = [[self.bookmarks objectAtIndex:row] name];
    
    // set the disclosure indicator...
	cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;

    return cell;
}

-(void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
	// find it...
	SBBookmark *bookmark = [self.bookmarks objectAtIndex:[indexPath row]];
	[self.owner openBookmarkSingleton:[bookmark ordinal]];
}


-(void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)style forRowAtIndexPath:(NSIndexPath *)indexPath
{
	// great - get rid of it...
	SBBookmark *bookmark = [self.bookmarks objectAtIndex:[indexPath row]];
	[bookmark setLocalDeleted:TRUE];
	[bookmark saveChanges];
	
	// update...
	[self stopEditing];
	[self refreshView];
}



- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)dealloc
{
    [syncEngine release];
    [bookmarks release];
    [table release];
    [super dealloc];
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

@end
