//
//  SBRestServiceProxy.m
//  SixBookmarks
//
//  Created by ss on 11. 5. 2..
//  Copyright 2011 OpenTheGate. All rights reserved.
//

#import "SBRestServiceProxy.h"
#import "MyClasses.h"

#import "SBErrorHelper.h"


@implementation SBRestServiceProxy

-(id)initWithServiceName:(NSString *)theServiceName
{
    return [super initWithServiceName:theServiceName];
}


-(void)makeRequest:(SBRestRequestArgs *)args opCode:(int)theOpCode
{
    //get the url...
    NSString *url = [SBHttpHelper buildUrl:[self resolvedServiceUrl] args:[args args]];
    
    //download...
    SBDownloadSettings *settings = [self getDownloadSettings];
    [SBHttpHelper download:url settings:settings callback:(SBDownloadCallback *)self opCode:theOpCode];
    
    //start the wheel spinning...
    [SBServiceProxy startSpinning];
}

-(void)downloadComplete:(SBDownloadBucket *)bucket
{
    //did we get a 200?
    NSError *err = nil;
    if (bucket.statusCode == 200)
    {
        NSXMLParser *parser = [[NSXMLParser alloc] initWithData:[bucket data]];
        
        //result...
        SBFlatXmlBucket *values = [[SBFlatXmlBucket alloc] init];
        [parser setDelegate:values];
        
        //run...
        [parser parse];
        
        //ok - what did we get?
        int result = [[values rootName] isEqualToString:@"AmxResponse"];
        if ([values rootName] == nil || !(result))
            err = [SBErrorHelper error:self message:[NSString stringWithFormat:@"The REST service '%@' returned a root element with anme '%@', not 'AmxResponse'.", [self serviceName], [values rootName]]];
        else
        {
            //did we get an exception?
            BOOL hasException = [values getBooleanVaule:@"HasException"];
            if (hasException)
            {
                NSString *message = [values getStringValue:@"Error"];
                err = [SBErrorHelper error:self message:[NSString stringWithFormat:@"The REST service '%@' returned an exception: '%@'.", serviceName, message]];
            }
            else
            {
                //good - now we can do something with it...
                err = [self processResult:values opCode:[bucket opCode]];
            }
        }
        
        //cleanup...
        [parser release];
        [values release];
    }
    else
        err = [SBErrorHelper error:self message:[NSString stringWithFormat:@"The server returned HTTP '%d'.", bucket.statusCode]];
    
    //stop the wheel spinning...
    [SBServiceProxy stopSpinning];
    
    
    //done that bit - now send a nitification...
    if (err == nil)
        [self requestOk:[bucket opCode]];
    else
        [self requestFailed:[bucket opCode] error:err];
}



-(NSError *)processResult:(SBFlatXmlBucket *)values opCode:(int)theOpCode
{
    return [SBErrorHelper error:self message:@"processResult handler not implemented."];
}


-(void)requestOk:(int)theOpCode
{
    //no-op by default...
}


-(void)requestFailed:(int)theOpCode error:(NSError *)theError
{
    //show an error...
    [SBMessageBox showError:theError];
}










@end
